# Databricks notebook source
# MAGIC %pip install dbtunnel[ray] transformers 
# MAGIC dbutils.library.restartPython()

# COMMAND ----------

ngrok_api_key: str = dbutils.secrets.get("fieldeng", "ngrok-api-key")
ngrok_tunnel_auth_token: str = dbutils.secrets.get("fieldeng", "ngrok-tunnel-auth-token")

# COMMAND ----------

from ray import serve
from ray.serve.deployment import Application

from transformers import pipeline

@serve.deployment(num_replicas=2, )
class SentimentAnalysis:

  def __init__(self):
    self._classifier = pipeline("sentiment-analysis")

  def __call__(self, request) -> str:
    input_text: str = request.query_params["input_text"]
    return self._classifier(input_text)[0]["label"]
  
  
app: Application = SentimentAnalysis.bind() #noqa: F821
 
  

# COMMAND ----------

from dbtunnel import dbtunnel

(
  dbtunnel.ray(app, port=8080)
    .inject_auth()
    .share_to_internet_via_ngrok(
        ngrok_api_token=ngrok_api_key, 
        ngrok_tunnel_auth_token=ngrok_tunnel_auth_token)
    .run()
)



from dbtunnel.tunnels import DbTunnel


class RayAppTunnel(DbTunnel):

    def __init__(self, ray, port: int = 8080):
        super().__init__(port, flavor="ray")
        self._ray = ray

    def _imports(self):
        try:
            from fastapi import FastAPI
            import ray
            import nest_asyncio
        except ImportError as e:
            self._log.info(
                "ImportError: Make sure you have ray, fastapi and nest_asyncio installed;"
                "pip install ray, fastapi nest_asyncio uvicorn"
            )
            raise e

    def _run(self):
        self.display()
        self._log.info("Starting server...")

        from ray import serve
        from ray.serve.handle import DeploymentHandle
        from ray.serve.config import HTTPOptions, ProxyLocation

        import nest_asyncio

        nest_asyncio.apply()

        async def start():
            http_options: HTTPOptions = HTTPOptions(host="0.0.0.0", port=self._port)
            proxy_location: ProxyLocation = ProxyLocation.HeadOnly
            serve.start(http_options=http_options, proxy_location=proxy_location)
            _: DeploymentHandle = serve.run(self._ray, route_prefix="/", blocking=True)

        import asyncio

        self._log.info(f"Use this link: \n{self._proxy_settings.proxy_url}")
        asyncio.run(start())

    def _display_url(self):
        # must end with a "/" for it to not redirect
        return f'<a href="{self._proxy_settings.proxy_url}">Click to go to {self._flavor} App!</a>'
